@@ -1,8 +1,9 @@
 use pyo3_build_config::{
     bail, ensure, print_feature_cfgs,
     pyo3_build_script_impl::{
-        cargo_env_var, env_var, errors::Result, is_linking_libpython, resolve_interpreter_config,
-        InterpreterConfig, PythonVersion,
+        cargo_env_var, env_var, errors::Result, is_linking_libpython_for_target,
+        resolve_build_config, target_triple_from_env, BuildConfig, BuildConfigSource,
+        InterpreterConfig, MaximumVersionExceeded, PythonVersion,
     },
     warn, PythonImplementation,
 };
@@ -17,16 +18,16 @@ const SUPPORTED_VERSIONS_CPYTHON: SupportedVersions = SupportedVersions {
     min: PythonVersion { major: 3, minor: 7 },
     max: PythonVersion {
         major: 3,
-        minor: 13,
+        minor: 14,
     },
 };
 
 const SUPPORTED_VERSIONS_PYPY: SupportedVersions = SupportedVersions {
-    min: PythonVersion { major: 3, minor: 7 },
-    max: PythonVersion {
+    min: PythonVersion {
         major: 3,
-        minor: 10,
+        minor: 11,
     },
+    max: SUPPORTED_VERSIONS_CPYTHON.max,
 };
 
 const SUPPORTED_VERSIONS_GRAALPY: SupportedVersions = SupportedVersions {
@@ -34,10 +35,7 @@ const SUPPORTED_VERSIONS_GRAALPY: SupportedVersions = SupportedVersions {
         major: 3,
         minor: 10,
     },
-    max: PythonVersion {
-        major: 3,
-        minor: 11,
-    },
+    max: SUPPORTED_VERSIONS_CPYTHON.max,
 };
 
 fn ensure_python_version(interpreter_config: &InterpreterConfig) -> Result<()> {
@@ -56,15 +54,32 @@ fn ensure_python_version(interpreter_config: &InterpreterConfig) -> Result<()> {
                 interpreter_config.version,
                 versions.min,
             );
-            ensure!(
-                interpreter_config.version <= versions.max || env_var("PYO3_USE_ABI3_FORWARD_COMPATIBILITY").map_or(false, |os_str| os_str == "1"),
-                "the configured Python interpreter version ({}) is newer than PyO3's maximum supported version ({})\n\
-                 = help: please check if an updated version of PyO3 is available. Current version: {}\n\
-                 = help: set PYO3_USE_ABI3_FORWARD_COMPATIBILITY=1 to suppress this check and build anyway using the stable ABI",
-                interpreter_config.version,
-                versions.max,
-                std::env::var("CARGO_PKG_VERSION").unwrap(),
-            );
+            let v_plus_1 = PythonVersion {
+                major: versions.max.major,
+                minor: versions.max.minor + 1,
+            };
+            if interpreter_config.version == v_plus_1 {
+                warn!(
+                    "Using experimental support for the Python {}.{} ABI. \
+                     Build artifacts may not be compatible with the final release of CPython, \
+                     so do not distribute them.",
+                    v_plus_1.major, v_plus_1.minor,
+                );
+            } else if interpreter_config.version > v_plus_1 {
+                let mut error = MaximumVersionExceeded::new(interpreter_config, versions.max);
+                if interpreter_config.is_free_threaded() {
+                    error.add_help(
+                        "the free-threaded build of CPython does not support the limited API so this check cannot be suppressed.",
+                    );
+                    return Err(error.finish().into());
+                }
+
+                if env_var("PYO3_USE_ABI3_FORWARD_COMPATIBILITY").is_none_or(|os_str| os_str != "1")
+                {
+                    error.add_help("set PYO3_USE_ABI3_FORWARD_COMPATIBILITY=1 to suppress this check and build anyway using the stable ABI");
+                    return Err(error.finish().into());
+                }
+            }
         }
         PythonImplementation::PyPy => {
             let versions = SUPPORTED_VERSIONS_PYPY;
@@ -75,14 +90,10 @@ fn ensure_python_version(interpreter_config: &InterpreterConfig) -> Result<()> {
                 versions.min,
             );
             // PyO3 does not support abi3, so we cannot offer forward compatibility
-            ensure!(
-                interpreter_config.version <= versions.max,
-                "the configured PyPy interpreter version ({}) is newer than PyO3's maximum supported version ({})\n\
-                 = help: please check if an updated version of PyO3 is available. Current version: {}",
-                interpreter_config.version,
-                versions.max,
-                std::env::var("CARGO_PKG_VERSION").unwrap()
-            );
+            if interpreter_config.version > versions.max {
+                let error = MaximumVersionExceeded::new(interpreter_config, versions.max);
+                return Err(error.finish().into());
+            }
         }
         PythonImplementation::GraalPy => {
             let versions = SUPPORTED_VERSIONS_GRAALPY;
@@ -93,23 +104,25 @@ fn ensure_python_version(interpreter_config: &InterpreterConfig) -> Result<()> {
                 versions.min,
             );
             // GraalPy does not support abi3, so we cannot offer forward compatibility
-            ensure!(
-                interpreter_config.version <= versions.max,
-                "the configured GraalPy interpreter version ({}) is newer than PyO3's maximum supported version ({})\n\
-                 = help: please check if an updated version of PyO3 is available. Current version: {}",
-                interpreter_config.version,
-                versions.max,
-                std::env::var("CARGO_PKG_VERSION").unwrap()
-            );
+            if interpreter_config.version > versions.max {
+                let error = MaximumVersionExceeded::new(interpreter_config, versions.max);
+                return Err(error.finish().into());
+            }
         }
     }
 
     if interpreter_config.abi3 {
         match interpreter_config.implementation {
-            PythonImplementation::CPython => {}
+            PythonImplementation::CPython => {
+                if interpreter_config.is_free_threaded() {
+                    warn!(
+                            "The free-threaded build of CPython does not yet support abi3 so the build artifacts will be version-specific."
+                    )
+                }
+            }
             PythonImplementation::PyPy => warn!(
                 "PyPy does not yet support abi3 so the build artifacts will be version-specific. \
-                See https://foss.heptapod.net/pypy/pypy/-/issues/3397 for more information."
+                See https://github.com/pypy/pypy/issues/3397 for more information."
             ),
             PythonImplementation::GraalPy => warn!(
                 "GraalPy does not support abi3 so the build artifacts will be version-specific."
@@ -142,7 +155,8 @@ fn ensure_target_pointer_width(interpreter_config: &InterpreterConfig) -> Result
     Ok(())
 }
 
-fn emit_link_config(interpreter_config: &InterpreterConfig) -> Result<()> {
+fn emit_link_config(build_config: &BuildConfig) -> Result<()> {
+    let interpreter_config = &build_config.interpreter_config;
     let target_os = cargo_env_var("CARGO_CFG_TARGET_OS").unwrap();
 
     println!(
@@ -163,7 +177,14 @@ fn emit_link_config(interpreter_config: &InterpreterConfig) -> Result<()> {
     );
 
     if let Some(lib_dir) = &interpreter_config.lib_dir {
-        println!("cargo:rustc-link-search=native={}", lib_dir);
+        println!("cargo:rustc-link-search=native={lib_dir}");
+    } else if matches!(build_config.source, BuildConfigSource::CrossCompile) {
+        warn!(
+            "The output binary will link to libpython, \
+            but PYO3_CROSS_LIB_DIR environment variable is not set. \
+            Ensure that the target Python library directory is \
+            in the rustc native library search path."
+        );
     }
 
     Ok(())
@@ -177,32 +198,35 @@ fn emit_link_config(interpreter_config: &InterpreterConfig) -> Result<()> {
 /// Emits the cargo configuration based on this config as well as a few checks of the Rust compiler
 /// version to enable features which aren't supported on MSRV.
 fn configure_pyo3() -> Result<()> {
-    let interpreter_config = resolve_interpreter_config()?;
+    let target = target_triple_from_env();
+    let build_config = resolve_build_config(&target)?;
+    let interpreter_config = &build_config.interpreter_config;
 
-    if env_var("PYO3_PRINT_CONFIG").map_or(false, |os_str| os_str == "1") {
-        print_config_and_exit(&interpreter_config);
+    if env_var("PYO3_PRINT_CONFIG").is_some_and(|os_str| os_str == "1") {
+        print_config_and_exit(interpreter_config);
     }
 
-    ensure_python_version(&interpreter_config)?;
-    ensure_target_pointer_width(&interpreter_config)?;
+    ensure_python_version(interpreter_config)?;
+    ensure_target_pointer_width(interpreter_config)?;
 
     // Serialize the whole interpreter config into DEP_PYTHON_PYO3_CONFIG env var.
     interpreter_config.to_cargo_dep_env()?;
 
-    if is_linking_libpython() && !interpreter_config.suppress_build_script_link_lines {
-        emit_link_config(&interpreter_config)?;
+    if is_linking_libpython_for_target(&target)
+        && !interpreter_config.suppress_build_script_link_lines
+    {
+        emit_link_config(&build_config)?;
     }
 
     for cfg in interpreter_config.build_script_outputs() {
-        println!("{}", cfg)
+        println!("{cfg}")
     }
 
     // Extra lines come last, to support last write wins.
     for line in &interpreter_config.extra_build_script_lines {
-        println!("{}", line);
+        println!("{line}");
     }
 
-    // Emit cfgs like `invalid_from_utf8_lint`
     print_feature_cfgs();
 
     Ok(())
