@@ -1,32 +1,32 @@
-// SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// SPDX-License-Identifier: MPL-2.0
+// Copyright ijl (2019-2026)
 
-use crate::str::PyStr;
+use crate::ffi::{Py_DECREF, Py_INCREF, PyStrRef};
 use associative_cache::{AssociativeCache, Capacity2048, HashDirectMapped, RoundRobinReplacement};
-use once_cell::unsync::OnceCell;
+use core::cell::OnceCell;
 
 #[repr(transparent)]
 pub(crate) struct CachedKey {
-    ptr: PyStr,
+    ptr: PyStrRef,
 }
 
 unsafe impl Send for CachedKey {}
 unsafe impl Sync for CachedKey {}
 
 impl CachedKey {
-    pub fn new(ptr: PyStr) -> CachedKey {
+    pub const fn new(ptr: PyStrRef) -> CachedKey {
         CachedKey { ptr: ptr }
     }
-    pub fn get(&mut self) -> PyStr {
+    pub fn get(&mut self) -> PyStrRef {
         let ptr = self.ptr.as_ptr();
-        debug_assert!(ffi!(Py_REFCNT(ptr)) >= 1);
-        ffi!(Py_INCREF(ptr));
-        self.ptr
+        unsafe { Py_INCREF(ptr) };
+        self.ptr.clone()
     }
 }
 
 impl Drop for CachedKey {
     fn drop(&mut self) {
-        ffi!(Py_DECREF(self.ptr.as_ptr().cast::<crate::ffi::PyObject>()));
+        unsafe { Py_DECREF(self.ptr.as_ptr().cast::<crate::ffi::PyObject>()) };
     }
 }
 
