@@ -1,4 +1,5 @@
-// SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// SPDX-License-Identifier: MPL-2.0
+// Copyright ijl (2018-2026)
 
 use crate::serialize::error::SerializeError;
 use crate::serialize::serializer::PyObjectSerializer;
@@ -29,16 +30,7 @@ impl Serialize for DefaultSerializer<'_> {
                     cold_path!();
                     err!(SerializeError::DefaultRecursionLimit)
                 }
-                #[cfg(not(Py_3_10))]
-                let default_obj = ffi!(PyObject_CallFunctionObjArgs(
-                    callable.as_ptr(),
-                    self.previous.ptr,
-                    core::ptr::null_mut::<crate::ffi::PyObject>()
-                ));
-                #[cfg(Py_3_10)]
-                #[allow(clippy::cast_sign_loss)]
-                let nargs = ffi!(PyVectorcall_NARGS(1)) as usize;
-                #[cfg(Py_3_10)]
+                let nargs = ffi!(PyVectorcall_NARGS(1)).cast_unsigned() as usize;
                 let default_obj = unsafe {
                     crate::ffi::PyObject_Vectorcall(
                         callable.as_ptr(),
