@@ -1,15 +1,17 @@
-// SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// SPDX-License-Identifier: MPL-2.0
+// Copyright ijl (2018-2026)
 
+use crate::ffi::PyFloatRef;
 use serde::ser::{Serialize, Serializer};
 
 #[repr(transparent)]
 pub(crate) struct FloatSerializer {
-    ptr: *mut crate::ffi::PyObject,
+    ob: PyFloatRef,
 }
 
 impl FloatSerializer {
-    pub fn new(ptr: *mut crate::ffi::PyObject) -> Self {
-        FloatSerializer { ptr: ptr }
+    pub fn new(ptr: PyFloatRef) -> Self {
+        FloatSerializer { ob: ptr }
     }
 }
 
@@ -19,6 +21,6 @@ impl Serialize for FloatSerializer {
     where
         S: Serializer,
     {
-        serializer.serialize_f64(ffi!(PyFloat_AS_DOUBLE(self.ptr)))
+        serializer.serialize_f64(self.ob.value())
     }
 }
