@@ -1,5 +1,6 @@
 # SPDX-License-Identifier: (Apache-2.0 OR MIT)
-
+# Copyright ijl (2020-2026), Ben Sully (2021), Nazar Kostetskyi (2022), Aviram Hassan (2020-2021), Marco Ribeiro (2020), Eric Jolibois (2021)
+# mypy: ignore-errors
 
 import sys
 
@@ -7,10 +8,7 @@ import pytest
 
 import orjson
 
-try:
-    import numpy
-except ImportError:
-    numpy = None  # type: ignore
+from .util import numpy
 
 
 def numpy_default(obj):
@@ -115,7 +113,7 @@ class TestNumpy:
                 numpy.array([1.0, 3.4028235e38], numpy.float32),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
-            == b"[1.0,3.4028235e38]"
+            == b"[1.0,3.4028235e+38]"
         )
 
     def test_numpy_array_d1_f16(self):
@@ -212,7 +210,7 @@ class TestNumpy:
                 numpy.array([1.0, 1.7976931348623157e308], numpy.float64),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
-            == b"[1.0,1.7976931348623157e308]"
+            == b"[1.0,1.7976931348623157e+308]"
         )
 
     def test_numpy_array_d1_bool(self):
@@ -237,7 +235,7 @@ class TestNumpy:
                         numpy.datetime64("2022"),
                         numpy.datetime64("2023"),
                         numpy.datetime64("9999"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -252,7 +250,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01"),
                         numpy.datetime64("2022-01"),
                         numpy.datetime64("2023-01"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -267,7 +265,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01"),
                         numpy.datetime64("2021-01-01"),
                         numpy.datetime64("2021-01-01"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -282,7 +280,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00"),
                         numpy.datetime64("2021-01-01T01"),
                         numpy.datetime64("2021-01-01T02"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -297,7 +295,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00"),
                         numpy.datetime64("2021-01-01T00:01"),
                         numpy.datetime64("2021-01-01T00:02"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -312,7 +310,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00:00"),
                         numpy.datetime64("2021-01-01T00:00:01"),
                         numpy.datetime64("2021-01-01T00:00:02"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -327,7 +325,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00:00"),
                         numpy.datetime64("2021-01-01T00:00:00.172"),
                         numpy.datetime64("2021-01-01T00:00:00.567"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -342,7 +340,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00:00"),
                         numpy.datetime64("2021-01-01T00:00:00.172"),
                         numpy.datetime64("2021-01-01T00:00:00.567891"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -357,7 +355,7 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00:00"),
                         numpy.datetime64("2021-01-01T00:00:00.172"),
                         numpy.datetime64("2021-01-01T00:00:00.567891234"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -372,11 +370,11 @@ class TestNumpy:
                         numpy.datetime64("2021-01-01T00:00:00"),
                         numpy.datetime64("2021-01-01T00:00:00.172"),
                         numpy.datetime64("2021-01-01T00:00:00.567891234567"),
-                    ]
+                    ],
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
-            assert False
+            raise AssertionError()
         except TypeError as exc:
             assert str(exc) == "unsupported numpy.datetime64 unit: picoseconds"
 
@@ -438,7 +436,8 @@ class TestNumpy:
         assert (
             orjson.dumps(
                 numpy.array(
-                    [[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]], numpy.float64
+                    [[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]],
+                    numpy.float64,
                 ),
                 option=orjson.OPT_SERIALIZE_NUMPY,
             )
@@ -451,7 +450,9 @@ class TestNumpy:
         with pytest.raises(orjson.JSONEncodeError):
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
         assert orjson.dumps(
-            array, default=numpy_default, option=orjson.OPT_SERIALIZE_NUMPY
+            array,
+            default=numpy_default,
+            option=orjson.OPT_SERIALIZE_NUMPY,
         ) == orjson.dumps(array.tolist())
 
     def test_numpy_array_non_contiguous_message(self):
@@ -459,7 +460,7 @@ class TestNumpy:
         assert array.flags["F_CONTIGUOUS"] is True
         try:
             orjson.dumps(array, option=orjson.OPT_SERIALIZE_NUMPY)
-            assert False
+            raise AssertionError()
         except TypeError as exc:
             assert (
                 str(exc)
@@ -479,7 +480,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -491,7 +492,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -503,7 +504,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -515,7 +516,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -527,7 +528,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -544,7 +545,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -555,7 +556,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -566,7 +567,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -612,7 +613,7 @@ class TestNumpy:
                 orjson.dumps(
                     array,
                     option=orjson.OPT_SERIALIZE_NUMPY,
-                )
+                ),
             )
             == array.tolist()
         )
@@ -651,13 +652,15 @@ class TestNumpy:
     def test_numpy_scalar_int64(self):
         assert (
             orjson.dumps(
-                numpy.int64(-9223372036854775808), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.int64(-9223372036854775808),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b"-9223372036854775808"
         )
         assert (
             orjson.dumps(
-                numpy.int64(9223372036854775807), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.int64(9223372036854775807),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b"9223372036854775807"
         )
@@ -686,7 +689,8 @@ class TestNumpy:
         assert orjson.dumps(numpy.uint64(0), option=orjson.OPT_SERIALIZE_NUMPY) == b"0"
         assert (
             orjson.dumps(
-                numpy.uint64(18446744073709551615), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.uint64(18446744073709551615),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b"18446744073709551615"
         )
@@ -733,7 +737,8 @@ class TestNumpy:
     def test_numpy_datetime_day(self):
         assert (
             orjson.dumps(
-                numpy.datetime64("2021-01-01"), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.datetime64("2021-01-01"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b'"2021-01-01T00:00:00"'
         )
@@ -741,7 +746,8 @@ class TestNumpy:
     def test_numpy_datetime_hour(self):
         assert (
             orjson.dumps(
-                numpy.datetime64("2021-01-01T00"), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.datetime64("2021-01-01T00"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b'"2021-01-01T00:00:00"'
         )
@@ -749,7 +755,8 @@ class TestNumpy:
     def test_numpy_datetime_minute(self):
         assert (
             orjson.dumps(
-                numpy.datetime64("2021-01-01T00:00"), option=orjson.OPT_SERIALIZE_NUMPY
+                numpy.datetime64("2021-01-01T00:00"),
+                option=orjson.OPT_SERIALIZE_NUMPY,
             )
             == b'"2021-01-01T00:00:00"'
         )
@@ -1059,7 +1066,7 @@ class TestNumpy:
 
     def test_numpy_repeated(self):
         data = numpy.array([[[1, 2], [3, 4], [5, 6], [7, 8]]], numpy.int64)  # type: ignore
-        for _ in range(0, 3):
+        for _ in range(3):
             assert (
                 orjson.dumps(
                     data,
@@ -1073,7 +1080,7 @@ class TestNumpy:
 class TestNumpyEquivalence:
     def _test(self, obj):
         assert orjson.dumps(obj, option=orjson.OPT_SERIALIZE_NUMPY) == orjson.dumps(
-            obj.tolist()
+            obj.tolist(),
         )
 
     def test_numpy_uint8(self):
@@ -1099,7 +1106,7 @@ class TestNumpyEquivalence:
 
     def test_numpy_int64(self):
         self._test(
-            numpy.array([-9223372036854775807, 9223372036854775807], numpy.int64)
+            numpy.array([-9223372036854775807, 9223372036854775807], numpy.int64),
         )
 
     @pytest.mark.skip(reason="tolist() conversion results in 3.4028234663852886e38")
@@ -1111,15 +1118,16 @@ class TestNumpyEquivalence:
                     340282346638528859811704183484516925440.0000000000000000,
                 ],
                 numpy.float32,
-            )
+            ),
         )
         self._test(numpy.array([-3.4028235e38, 3.4028235e38], numpy.float32))
 
     def test_numpy_float64(self):
         self._test(
             numpy.array(
-                [-1.7976931348623157e308, 1.7976931348623157e308], numpy.float64
-            )
+                [-1.7976931348623157e308, 1.7976931348623157e308],
+                numpy.float64,
+            ),
         )
 
 
