@@ -1,10 +1,16 @@
-# SPDX-License-Identifier: (Apache-2.0 OR MIT)
+# SPDX-License-Identifier: MPL-2.0
+# Copyright ijl (2018-2026)
 
 import pytest
 
 import orjson
 
-from .util import needs_data, read_fixture_bytes
+from .util import (
+    SUPPORTS_BYTEARRAY,
+    SUPPORTS_MEMORYVIEW,
+    needs_data,
+    read_fixture_bytes,
+)
 
 
 @needs_data
@@ -13,10 +19,12 @@ class TestJSONTestSuiteParsing:
         data = read_fixture_bytes(filename, "parsing")
         with pytest.raises(exc):
             orjson.loads(data)
-        with pytest.raises(exc):
-            orjson.loads(bytearray(data))
-        with pytest.raises(exc):
-            orjson.loads(memoryview(data))
+        if SUPPORTS_BYTEARRAY:
+            with pytest.raises(exc):
+                orjson.loads(bytearray(data))
+        if SUPPORTS_MEMORYVIEW:
+            with pytest.raises(exc):
+                orjson.loads(memoryview(data))
         try:
             decoded = data.decode("utf-8")
         except UnicodeDecodeError:
@@ -28,8 +36,10 @@ class TestJSONTestSuiteParsing:
     def _run_pass_json(self, filename, match=""):
         data = read_fixture_bytes(filename, "parsing")
         orjson.loads(data)
-        orjson.loads(bytearray(data))
-        orjson.loads(memoryview(data))
+        if SUPPORTS_BYTEARRAY:
+            orjson.loads(bytearray(data))
+        if SUPPORTS_MEMORYVIEW:
+            orjson.loads(memoryview(data))
         orjson.loads(data.decode("utf-8"))
 
     def test_y_array_arraysWithSpace(self):
