@@ -1,9 +1,10 @@
-# SPDX-License-Identifier: (Apache-2.0 OR MIT)
+# SPDX-License-Identifier: MPL-2.0
+# Copyright ijl (2019-2023)
 
 import orjson
 
 try:
-    from typing import TypedDict
+    from typing import TypedDict  # type: ignore
 except ImportError:
     from typing_extensions import TypedDict
 
